import {
  Modal, ModalService, Button,
  GridEditable, IButtonEvent, DialogService,
  Text, Form, IComponentRender,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00608ConfiguracaoCardapio {
  private modalConfigCard!: Modal;
  
  public utilities = Loader.getInstance('Utilities');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  private gridEditableController = Loader.getInstance('GridEditableController');
  
  public modalController = Loader.getInstance('ModalController');
  
  private maxSeq = 0;
  
  private maxOrd = 0;
  
  public async createModal() {
    this.modalConfigCard = await ModalService.createFromJson('pme/cardapio/modals/pme00608_configuracao_cardapio', true);
  }
  
  public cancelModal() {
    this.modalConfigCard.hide();
    // this.modalConfigCard.destroy();
    this.elaboracaoCardapio.reloadCard();
  }
  
  public showModalConfigCard({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    this.validAndOpenModal();
  }
  
  public async validAndOpenModal() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    if (cardapio.dateLessDateOfProcess(cardapio.dates.slice(-1)[0].DTCARDIAPME)) {
      if (this.elaboracaoCardapio.getFilterData.NRCARDPME !== undefined) {
        if (this.modalConfigCard) {
          this.modalConfigCard.show();
        } else {
          this.createModal().then(() => {
            this.modalConfigCard.show();
          });
        }
      } else {
        this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
      }
    } else {
      this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
    }
  }
  
  private mountDescOnEnterConfig() {
    Metadata.getInstance<Text>('valueTextService').text += this.elaboracaoCardapio.getFilterData.service.DESCRICAO;
    Metadata.getInstance<Text>('valuePeriodDate').text += this.elaboracaoCardapio.getFilterData.initialDate;
    Metadata.getInstance<Text>('valuePeriodDate').text += 'pme::ate';
    Metadata.getInstance<Text>('valuePeriodDate').text += this.elaboracaoCardapio.getFilterData.finalDate;
  }
  
  public filterGridConfCard() {
    const gridConf = Metadata.getInstance<GridEditable>('dragDropConfigCard');
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.elaboracaoCardapio.getFilterData.cdfilcentpme);
    this.utilities.onMountedFilterFind(
      filter, 'P_CDSERVFILI', '=', this.elaboracaoCardapio.getFilterData.service.CDSERVFILI,
    );
    this.utilities.onMountedFilterFind(
      filter, 'P_CDTIPOCARD', '=', this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD,
    );
    this.utilities.onMountedFilterFind(filter, 'P_DTINICARDPME', '=', this.elaboracaoCardapio.getFilterData.initialDate);
    this.utilities.onMountedFilterFind(filter, 'P_DTFINCARDPME', '=', this.elaboracaoCardapio.getFilterData.finalDate);
    this.utilities.onMountedFilterFind(filter, 'P_NRCARDPME', '=', this.elaboracaoCardapio.getFilterData.NRCARDPME);
    
    gridConf.datasource.filter = filter;
    this.fixGridButtons(gridConf);
    gridConf.datasource.get();
  }
  
  private fixGridButtons(grid: GridEditable) {
    const toolbarSlot = grid.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton' && e.disabled === false) e.disabled = !e.disabled;
      if (e.name === 'saveButton' && e.disabled === false) e.disabled = !e.disabled;
    });
  }
  
  public getMaxSeq() {
    const row = this.elaboracaoCardapio.getFilterData;
    Http.post('/configCard/get', { requestType: 'Row', row }).then((response) => {
      this.maxSeq = response.data.dataset.maxSeq[0]?.NRSEQCNFCPME;
      this.maxOrd = response.data.dataset.maxSeq[0]?.NRORDCNFCPME;
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  // Função retirada do GridEditableController e modificar para o comportamento da tela...
  public addRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    if (!this.gridEditableController.newRow) {
      this.maxSeq += 1;
      this.maxOrd += 1;
      grid.datasource.data.push({
        __is_new: true,
        IDOPCCNFCPME: 'N',
        IDIMPCNFCPME: 'S',
        NRORDCNFCPME: this.maxOrd,
        NRSEQCNFCPME: this.maxSeq,
        KEY: `${this.elaboracaoCardapio.getFilterData.cdfilcentpme}
          ${this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD}
          ${this.elaboracaoCardapio.getFilterData.service.CDSERVFILI}
          ${this.maxOrd}${'N'}${'S'}${this.maxOrd}`,
      });
      this.gridEditableController.newRow = true;
      if (!grid.editing) {
        grid.editing = true;
        if (!this.gridEditableController.isInlineEditing) this.gridEditableController.onInlineEdit({ component: grid });
      }
    }
  }
  
  public saveConfigCard({ component }: IButtonEvent) {
    const formGrid = Metadata.getInstance<Form>('formDragGrid');
    if (formGrid.validate()) {
      const grid: GridEditable = component.parent as GridEditable;
      if (grid.getEditedRows().length === 0) {
        this.utilities.showAlert('alertWarning', 'pme::nenhumRegEditado', 'warning');
      } else {
        const row: IDictionary = {};
        row.editedRowsCard = grid.getEditedRows();
        row.filterData = this.elaboracaoCardapio.getFilterData;
        Http.post('/configCard/save', { requestType: 'Row', row }).then((response) => {
          if (response.data.dataset.saveConfigCard[0]) {
            this.maxSeq += 1;
            this.maxOrd += 1;
            this.gridEditableController.defaultState(grid).then(() => {
              this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
            });
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
  
  public confirmDeleteConfCard() {
    const grid = Metadata.getInstance<GridEditable>('dragDropConfigCard');
    const message = (grid.selectedRows.length > 1) ? 'pme::desejaDeletaConfCardPlural' : 'pme::desejaDeletaConfCard';
    if (grid.selectedRows.length === 0) {
      this.utilities.showAlert('alertWarning', 'pme::nenhumRegSelecionado', 'warning');
    } else {
      DialogService.show({
        name: 'confirmDeleteConfCard',
        type: 'warning',
        title: 'pme::atencao',
        text: message,
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteConfCard() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  private deleteConfCard() {
    const button = Metadata.getInstance<Button>('yes');
    const grid = Metadata.getInstance<GridEditable>('dragDropConfigCard');
    const row = grid.selectedRows;
    this.utilities.activeLoadingButton(button);
    Http.post('/configCard/delete', { requestType: 'Row', row }).then((response) => {
      if (response.data.dataset.deleteConfigCard[0]) {
        grid.datasource.get();
        DialogService.hide();
        grid.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
        this.utilities.disableLoadingButton(button);
      }
    }).catch((error: string) => {
      DialogService.hide();
      grid.selectedRows = [];
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
}
